package question;

import answer.Answer;

import java.util.Vector;

public class Question {
    
    public Question(
            String pContent,
            String pExplanation,
            int pCost,
            int pPenalty,
            Vector< Answer > pAnswers )
            throws IllegalArgumentException {
        
        if( pContent == null ) {
            throw new IllegalArgumentException( "Illegal Content argument." );
        }
        if( pContent.trim().length() == 0 ) {
            throw new IllegalArgumentException( "Illegal Content argument." );
        }
        if( pExplanation == null ) {
            throw new IllegalArgumentException( "Illegal Explanation argument." );
        }
        if( pExplanation.trim().length() == 0 ) {
            throw new IllegalArgumentException( "Illegal Explanation argument." );
        }
        if( pAnswers == null ) {
            throw new IllegalArgumentException( "Illegal Answers argument." );
        }
        if( pAnswers.size() == 0 ) {
            throw new IllegalArgumentException( "Illegal Answers argument." );
        }
        
        mContent = new String( pContent );
        mExplanation = new String( pExplanation );
        setCost( pCost );
        setPenalty( pPenalty );
        mAnswers = ( Vector< Answer > )pAnswers.clone();
    }
    
    public Question( 
            Question pPrototype )
            throws IllegalArgumentException {
        
        if( pPrototype == null ) {
            throw new IllegalArgumentException( "Illegal Prototype argument." );
        }
        mContent = new String( pPrototype.mContent );
        mExplanation = new String( pPrototype.mExplanation );
        setCost( pPrototype.mCost );
        setPenalty( pPrototype.mPenalty );
        mAnswers = ( Vector< Answer > )pPrototype.mAnswers.clone();
    }
    
    public boolean setContent(
            String pContent ) {
        
        if( pContent == null )  {
            return false;
        }
        if( pContent.trim().length() == 0 ) {
            return false;
        }
        mContent = new String( pContent );
        return true;
    }
    
    public String getContent() {
        
        return new String( mContent );
    }
    
    public boolean setExplanation(
            String pExplanation ) {
        
        if( pExplanation == null )  {
            return false;
        }
        if( pExplanation.trim().length() == 0 ) {
            return false;
        }
        mExplanation = new String( pExplanation );
        return true;
    }
    
    public String getExplanation() {
        
        return new String( mExplanation );
    }
    
    public void setCost( 
            int pCost ) {
        
        if( pCost < cPercentageLowBound ) {
            mCost = cPercentageLowBound;
        } else if( pCost > cPercentageHighBound ) {
            mCost = cPercentageHighBound;
        } else {
            mCost = pCost;
        }
    }
    
    public int getCost() {
        
        return mCost;
    }
    
    public void setPenalty(
            int pPenalty ) {
        
        if( pPenalty < cPercentageLowBound ) {
            mPenalty = cPercentageLowBound;
        } else if( pPenalty > cPercentageHighBound ) {
            mPenalty = cPercentageHighBound;
        } else {
            mPenalty = pPenalty;
        }
    }
    
    public int getPenalty() {
        
        return mPenalty;
    }
    
    public boolean setAnswers(
            Vector< Answer > pAnswers ) {
        
        if( pAnswers == null ) {
            return false;
        } else if( pAnswers.size() == 0 ) {
            return false;
        } else {
            mAnswers = ( Vector< Answer > )pAnswers.clone();
            return true;
        }
    }
    
    public Vector< Answer > getAnswers() {
        
        return ( Vector< Answer > )mAnswers.clone();
    }
    
    public Question clone() {
        
        Question lClone = null;
        try {
            lClone = new Question( this );
        } catch( IllegalArgumentException eFake ) {
            
        }
        return lClone;
    }
    
    public boolean equals(
            Object pInstance ) {
        
        if( pInstance == null ) {
            return false;
        } else if( pInstance instanceof Question ) {
            Question lInstance = ( Question )pInstance;
            if( ( mContent.equals( lInstance.mContent ) ) &&
                    ( mExplanation.equals( lInstance.mExplanation ) ) &&
                    ( mAnswers.equals( lInstance.mAnswers ) ) ) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
    
    private String mContent;
    private String mExplanation;
    private int mCost;
    private int mPenalty;
    protected Vector< Answer > mAnswers;
    
    private static final int cPercentageLowBound = 0;
    private static final int cPercentageHighBound = 100;
}