package test;

import department.Department;
import author.Author;

import java.util.Date;
import java.util.Vector;

public class Description {
    
    public Description(
            Department pDepartment,
            Vector< Author > pAuthors,
            Author pEditor,
            Author pReviewer,
            Date pCreationDate,
            Date pPublicationDate,
            Date pRenovationDate )
            throws IllegalArgumentException {
        
        if( pAuthors == null ) {
            throw new IllegalArgumentException( "Illegal Authors argument." );
        }
        if( pAuthors.size() == 0 ) {
            throw new IllegalArgumentException( "Illegal Authors argument." );
        }
        if( pCreationDate == null ) {
            throw new IllegalArgumentException( "Illegal CreationDate argument." );
        }
        if( pPublicationDate == null ) {
            throw new IllegalArgumentException( "Illegal PublicationDate argument." );
        }
        if( pPublicationDate.before( pCreationDate) ) {
            throw new IllegalArgumentException( "Illegal PublicationDate argument." );
        }
        if( pRenovationDate == null ) {
            throw new IllegalArgumentException( "Illegal RenovationDate argument." );
        }
        if( pRenovationDate.before( pPublicationDate ) ) {
            throw new IllegalArgumentException( "Illegal RenovationDate argument." );
        }
        
        setDepartment( pDepartment );
        mAuthors = ( Vector< Author > )pAuthors.clone();
        setEditor( pEditor );
        setReviewer( pReviewer );
        mCreationDate = ( Date )pCreationDate.clone();
        mPublicationDate = ( Date )pPublicationDate.clone();
        mRenovationDate = ( Date )pRenovationDate.clone();
    }
    
    public Description(
            Description pPrototype )
            throws IllegalArgumentException {
        
        if( pPrototype == null ) {
            throw new IllegalArgumentException( "Illegal Prototype argument." );
        }
        setDepartment( pPrototype.mDepartment );
        mAuthors = ( Vector< Author > )pPrototype.mAuthors.clone();
        setEditor( pPrototype.mEditor );
        setReviewer( pPrototype.mReviewer );
        mCreationDate = ( Date )pPrototype.mCreationDate.clone();
        mPublicationDate = ( Date )pPrototype.mPublicationDate.clone();
        mRenovationDate = ( Date )pPrototype.mRenovationDate.clone();
    }
    
    public void setDepartment(
            Department pDepartment ) {
        
        mDepartment = pDepartment;
    }
    
    public Department getDepartment() {
        
        return mDepartment;
    }
    
    public boolean setAuthors( 
            Vector< Author > pAuthors ) {
        
        if( pAuthors == null ) {
            return false;
        } else if( pAuthors.size() == 0 ) {
            return false;
        } else {
            mAuthors = ( Vector< Author > )pAuthors.clone();
            return true;
        }
    }
    
    public Vector< Author > getAuthors() {
        
        return ( Vector< Author > )mAuthors.clone();
    }
    
    public void setEditor(
            Author pEditor ) {
        
        mEditor = pEditor;
    }
    
    public Author getEditor() {
        
        return mEditor;
    }
    
    public void setReviewer(
            Author pReviewer ) {
        
        mReviewer = pReviewer;
    }
    
    public Author getReviewer() {
        
        return mReviewer;
    }
    
    public boolean setCreationDate(
            Date pCreationDate ) {
        
        if( pCreationDate == null ) {
            return false;
        } else if( pCreationDate.after( mPublicationDate ) ) {
            return false;
        } else {
            mCreationDate = ( Date )pCreationDate.clone();
            return true;
        }
    }
    
    public Date getCreationDate() {
        
        return ( Date )mCreationDate.clone();
    }
    
    public boolean setPublicationDate(
            Date pPublicationDate ) {
        
        if( pPublicationDate == null ) {
            return false;
        } else if( pPublicationDate.before( mCreationDate ) ) {
            return false;
        } else if( pPublicationDate.after( mRenovationDate ) ) {
            return false;
        } else {
            mPublicationDate = ( Date )pPublicationDate.clone();
            return true;
        }
    }
    
    public Date getPublicationDate() {
        
        return ( Date )mPublicationDate.clone();
    }
    
    public boolean setRenovationDate(
            Date pRenovationDate ) {
        
        if( pRenovationDate == null ) {
            return false;
        } else if( pRenovationDate.before( mPublicationDate ) ) {
            return false;
        } else {
            mRenovationDate = ( Date )pRenovationDate.clone();
            return true;
        }
    }
    
    public Date getRenovationDate() {
        
        return ( Date )mRenovationDate.clone();
    }
    
    public Description clone() {
        
        Description lClone = null;
        try {
            lClone = new Description( this );
        } catch ( IllegalArgumentException eFake ) {
            
        }
        return lClone;
    }
    
    public boolean equals( 
            Object pInstance ) {
        
        if( pInstance == null ) {
            return false;
        } else if( pInstance instanceof Description ) {
            Description lInstance = ( Description )pInstance;
            if( ( mDepartment.equals( lInstance.mDepartment ) ) &&
                    ( mAuthors.equals( lInstance.mAuthors) ) &&
                    ( mEditor.equals( lInstance.mEditor) ) &&
                    ( mReviewer.equals( lInstance.mReviewer ) ) &&
                    ( mCreationDate.equals( lInstance.mCreationDate) ) &&
                    ( mPublicationDate.equals( lInstance.mPublicationDate ) ) &&
                    ( mRenovationDate.equals( lInstance.mRenovationDate ) ) ) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }
    
    private Department mDepartment;
    private Vector< Author > mAuthors;
    private Author mEditor;
    private Author mReviewer;
    private Date mCreationDate;
    private Date mPublicationDate;
    private Date mRenovationDate;
}